/*
 * Copyright (c) 2004 Michael Schroeder (mls@suse.de)
 *
 * This program is licensed under the BSD license, read LICENSE.BSD
 * for further information
 */

#include <fcntl.h>
#include <string.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <rpmrc.h>
#include <rpmtypes.h>
#include <rpmtag.h>
#include <rpmlog.h>
#include <rpmio.h>
#include <rpmts.h>
#include <rpmdb.h>
#include <pkgio.h>

int main(int argc, char **argv)
{
  FD_t fdo;
  rpmts ts = NULL;
  rpmmi mi;
  Header h;
  int ret = 0;

  char *n;
  char *e;
  char *v;
  char *r;
  char *a = NULL;

  if (argc > 2 && !strcmp(argv[1], "-a"))
    {
      a = argv[2];
      argc -= 2;
      argv += 2;
    }
  if (argc != 2)
    {
      fprintf(stderr, "usage: rpmdumpheader package\n");
      exit(1);
    }
  n = argv[1];
  r = strrchr(n, '-');
  if (r == 0 || r == n)
    {
      fprintf(stderr, "package name must be in n-(e:)v-r format\n");
      exit(1);
    }
  *r++ = 0;
  v = strrchr(n, '-');
  if (v == 0 || v == n)
    {
      fprintf(stderr, "package name must be in n-(e:)v-r format\n");
      exit(1);
    }
  *v++ = 0;
  e = strchr(v, ':');
  if (e)
    {
      char *t;
      *e++ = 0;
      t = e;
      e = v;
      v = t;
    }

  rpmReadConfigFiles(NULL, NULL);

  ts = rpmtsCreate();
  
  mi = rpmtsInitIterator(ts, RPMTAG_NAME, n, 0);
  rpmmiAddPattern(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP, e);
  rpmmiAddPattern(mi, RPMTAG_VERSION, RPMMIRE_STRCMP, v);
  rpmmiAddPattern(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP, r);
  if (a)
    rpmmiAddPattern(mi, RPMTAG_ARCH, RPMMIRE_STRCMP, a);

  if ((h = rpmmiNext(mi)) != NULL)
    {
	rpmRC rc;
	const char item[] = "Header";
	const char * msg = NULL;

      	fdo = Fopen("-", "w.ufdio");
	rc = rpmpkgWrite(item, (FD_t)fdo, h, &msg);
	if (rc != RPMRC_OK) {
		rpmlog(RPMLOG_ERR, "%s: %s: %s\n", "headerWrite", item, msg);
	}
	msg = (const char*)_free(msg);
    }
  else
    {
      if (e)
	fprintf(stderr, "%s-%s:%s-%s%s%s is not installed\n", n, e, v, r, a ? "." : "", a ? a : "");
      else
	fprintf(stderr, "%s-%s-%s%s%s is not installed\n", n, v, r, a ? "." : "", a ? a : "");
      ret = 1;
    }
  mi = rpmmiFree(mi);
  ts = rpmtsFree(ts);
  exit(ret);
}
